from typing import Dict, Optional, Generator
from .tools.typing.response import UseslessResponse
import requests
import json

class Model:
	@classmethod
	def __init__(self: type, model: Optional[str] = "gpt-3.5-turbo", temperature: Optional[int] = 1) -> None:
		self.__session: requests.Session = requests.Session()
		self.__JSON: Dict[str, str] = {"openaiKey": "", "prompt": "", "options": self.__SetOptions(model=model, 
			temperature=temperature, presence_penalty=0.8)}

		self.__HEADERS: Dict[str, str] = {
			"Authority": "ai.usesless.com",
			"Accept": "*/*",
			"Accept-Language": "pt-BR,en-US,en;q=0;5",
			"Origin": "https://ai.usesless.com",
			"Referer": "https://ai.usesless.com/chat",
			"Cache-Control": "no-cache",
			"Sec-Fetch-Dest": "empty",
			"Sec-Fetch-Mode": "cors",
			"Sec-Fetch-Site": "same-origin",
			"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/112.0"
		}

	@classmethod
	def __SetOptions(self: type, **kwargs) -> Dict[str, str]:
		return {"completionParams": kwargs, "systemMessage": "You are ChatGPT, a large language model trained by OpenAI. Answer as concisely as possible."}

	@classmethod
	def SetupConversation(self: type, prompt: str) -> None:
		self.__JSON["prompt"] = prompt

	@classmethod
	def SendConversation(self: type) -> Generator[UseslessResponse, None, None]:
		for chunk in self.__session.post("https://ai.usesless.com/api/chat-process", headers=self.__HEADERS, json=self.__JSON, stream=True).iter_lines():
			data = json.loads(chunk.decode("utf-8"))
			yield UseslessResponse(**data["detail"])


from Crypto.Cipher import AES
from Crypto.Random import get_random_bytes
from base64 import b64encode, b64decode
import hashlib

def Encrypt(data: b64encode, key: str) -> bytes:
    hash_key: hashlib.sha256 = hashlib.sha256(key.encode()).digest()
    iv: bytes = get_random_bytes(16)
    cipher: AES = AES.new(hash_key, AES.MODE_CBC, iv)
    encrypted_data: cipher.encrypt = cipher.encrypt(PadData(data.encode()))
    return iv.hex() + encrypted_data.hex()

def PadData(data):
    block_size: int = AES.block_size
    padding_size: int = block_size - len(data) % block_size
    padding: bytes = bytes([padding_size] * padding_size)
    return data + padding


<<<<<<< HEAD:opengpt/models/completion/forefront/tools/system/email_creation.py
from typing import Dict
from libraries.tempmail import TempMail
from libraries.colorama import init, Fore, Style
from ..typing.response import EmailResponse
import re
import json
import logging
import fake_useragent
import tls_client

init()

class Email:
	@classmethod
	def __init__(self: type) -> None:
		self.__SETUP_LOGGER()
		self.__session: tls_client.Session = tls_client.Session(client_identifier="chrome_110")

	@classmethod
	def __SETUP_LOGGER(self: type) -> None:
		self.__logger: logging.getLogger = logging.getLogger(__name__)
		self.__logger.setLevel(logging.DEBUG)
		console_handler: logging.StreamHandler = logging.StreamHandler()
		console_handler.setLevel(logging.DEBUG)
		formatter: logging.Formatter = logging.Formatter("Account Creaction - %(levelname)s - %(message)s")
		console_handler.setFormatter(formatter)

		self.__logger.addHandler(console_handler)

	@classmethod
	def __AccountState(self: object, output: str, field: str) -> bool:
		if field not in output:
			return False
		return True

	@classmethod
	def CreateAccount(self: object) -> str:
		Mail = TempMail()
		MailAddress = Mail.GetAddress

		self.__session.headers = {
			"Origin": "https://accounts.forefront.ai",
			"User-Agent": fake_useragent.UserAgent().random
		}

		self.__logger.debug(Fore.CYAN + "Checking URL" + Style.RESET_ALL)
		
		output = self.__session.post("https://clerk.forefront.ai/v1/client/sign_ups?_clerk_js_version=4.38.4", data={"email_address": MailAddress})

		if not self.__AccountState(str(output.text), "id"):
			self.__logger.error(Fore.RED + "Failed to create account." + Style.RESET_ALL)
			return "Failed"

		trace_id = output.json()["response"]["id"]

		output = self.__session.post(f"https://clerk.forefront.ai/v1/client/sign_ups/{trace_id}/prepare_verification?_clerk_js_version=4.38.4", 
			data={"strategy": "email_link", "redirect_url": "https://accounts.forefront.ai/sign-up/verify"})

		if not self.__AccountState(output.text, "sign_up_attempt"):
			self.__logger.error(Fore.RED + "Failed to create account." + Style.RESET_ALL)
			return "Failed"

		self.__logger.debug(Fore.CYAN + "Verifying account" + Style.RESET_ALL)

		while True:
			messages: Mail.GetMessages = Mail.GetMessages()

			if len(messages) > 0:
				message: Dict[str, str] = Mail.GetMessage(messages[0]["_id"])
				verification_url = re.findall(r"https:\/\/clerk\.forefront\.ai\/v1\/verify\?token=\w.+", message["bodyHtml"])[0]
				if verification_url:
					break

		r = self.__session.get(verification_url.split("\"")[0])
		__client: str = r.cookies["__client"]

		output = self.__session.get("https://clerk.forefront.ai/v1/client?_clerk_js_version=4.38.4")
		token: str = output.json()["response"]["sessions"][0]["last_active_token"]["jwt"]
		sessionID: str = output.json()["response"]["last_active_session_id"]

		self.__logger.debug(Fore.GREEN + "Created account!" + Style.RESET_ALL)

=======
from typing import Dict
from .tempmail import TempMail
from ..typing.response import EmailResponse
import re
import json
import logging
import fake_useragent
import tls_client

class Email:
	@classmethod
	def __init__(self: type) -> None:
		self.__SETUP_LOGGER()
		self.__session: tls_client.Session = tls_client.Session(client_identifier="chrome_110")

	@classmethod
	def __SETUP_LOGGER(self: type) -> None:
		self.__logger: logging.getLogger = logging.getLogger(__name__)
		self.__logger.setLevel(logging.DEBUG)
		console_handler: logging.StreamHandler = logging.StreamHandler()
		console_handler.setLevel(logging.DEBUG)
		formatter: logging.Formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
		console_handler.setFormatter(formatter)

		self.__logger.addHandler(console_handler)

	@classmethod
	def __AccountState(self: object, output: str, field: str) -> bool:
		if field not in output:
			return False
		return True

	@classmethod
	def CreateAccount(self: object) -> str:
		Mail = TempMail()
		MailAddress = Mail.GetAddress

		self.__session.headers = {
			"Origin": "https://accounts.forefront.ai",
			"User-Agent": fake_useragent.UserAgent().random
		}

		self.__logger.debug("Checking URL")
		
		output = self.__session.post("https://clerk.forefront.ai/v1/client/sign_ups?_clerk_js_version=4.38.4", data={"email_address": MailAddress})

		if not self.__AccountState(str(output.text), "id"):
			self.__logger.error("Failed to create account :(")
			return "Failed"

		trace_id = output.json()["response"]["id"]

		output = self.__session.post(f"https://clerk.forefront.ai/v1/client/sign_ups/{trace_id}/prepare_verification?_clerk_js_version=4.38.4", 
			data={"strategy": "email_link", "redirect_url": "https://accounts.forefront.ai/sign-up/verify"})

		if not self.__AccountState(output.text, "sign_up_attempt"):
			self.__logger.error("Failed to create account :(")
			return "Failed"

		self.__logger.debug("Verifying account")

		while True:
			messages: Mail.GetMessages = Mail.GetMessages()

			if len(messages) > 0:
				message: Dict[str, str] = Mail.GetMessage(messages[0]["_id"])
				verification_url = re.findall(r"https:\/\/clerk\.forefront\.ai\/v1\/verify\?token=\w.+", message["bodyHtml"])[0]
				if verification_url:
					break

		r = self.__session.get(verification_url.split("\"")[0])
		__client: str = r.cookies["__client"]

		output = self.__session.get("https://clerk.forefront.ai/v1/client?_clerk_js_version=4.38.4")
		token: str = output.json()["response"]["sessions"][0]["last_active_token"]["jwt"]
		sessionID: str = output.json()["response"]["last_active_session_id"]

		self.__logger.debug("Created account!")

>>>>>>> 79cab608409c040e5ef9b9496ef0fce35d025764:opengpt/forefront/tools/system/email_creation.py
		return EmailResponse(**{"sessionID": sessionID, "client": __client})


from pydantic import BaseModel
from typing import List, Optional, Union

class DeltaResponse(BaseModel):
	content: Optional[Union[str, None]] = ''
	role: Optional[Union[str, None]] = ''

class ChoicesResponse(BaseModel):
	delta: DeltaResponse
	index: int
	finish_reason: Optional[Union[str, None]] = ''

class UseslessResponse(BaseModel):
	id: str
	object: str
	created: int
	model: str
	choices: List[ChoicesResponse]


# <Library by UesleiDev />
# <w> Thanks to Use! </w>

from typing import Dict, List, Union
import tls_client
import fake_useragent

class TempMail:
	@classmethod 
	def __init__(self: type) -> None:
		self.__api: str = "https://web2.temp-mail.org"
		self.__session: tls_client.Session = tls_client.Session(client_identifier="chrome_110")

		self.__HEADERS: Dict[str, str] = {
			"Authority": "web2.temp-mail.org",
			"Accept": "*/*",
			"Accept-Language": "pt-BR,en;q=0.9,en-US;q=0.8,en;q=0.7",
			"Authorization": f"Bearer {self.__GetTokenJWT()}",
			"Origin": "https://temp-mail.org",
			"Referer": "https://temp-mail.org/",
			"Sec-Ch-Ua": "\"Chromium\";v=\"112\", \"Google Chrome\";v=\"112\", \"Not:A-Brand\";v=\"99\"",
			"Sec-Ch-Ua-mobile": "?0",
			"Sec-Ch-Ua-platform": "\"macOS\"",
			"Sec-Fetch-Dest": "empty",
			"Sec-Fetch-Mode": "cors",
			"Sec-Fetch-Site": "same-site",
			"User-Agent": fake_useragent.UserAgent().random
		}

	@classmethod
	def __GetTokenJWT(self: type) -> str:
		DATA_: Dict[str, str] = self.__session.post(f"{self.__api}/mailbox").json()

		self.__EMAIL: str = DATA_["mailbox"]
		return DATA_["token"]

	@property
	def GetAddress(self: type) -> str:
		return f"{self.__EMAIL}"

	@classmethod
	def GetMessages(self: type) -> List[Dict[str, str]]:
		messages: Union[List, List[Dict[str, str]]] = []

		messages = self.__session.get(f"{self.__api}/messages", headers=self.__HEADERS).json()["messages"]

		return messages

	@classmethod
	def GetMessage(self: type, id: str) -> Dict[str, str]:
		DATA_: object = self.__session.get(f"{self.__api}/messages/{id}", headers=self.__HEADERS)

		if DATA_.status_code != 200:
			return "Invalid ID."

		return DATA_.json()


from typing import Text
import random
import string

def UniqueID(size: int) -> Text:
    characters: Text = string.ascii_letters + string.digits
    id_: Text = ''.join(random.choice(characters) for _ in range(size))
    return id_


from typing import Optional, Union, Dict, List, Text
from libraries.colorama import init, Fore, Style
import importlib
import yaml
import sys
import inspect
import os

init()

class OpenGPTError(Exception):
	@staticmethod
	def Print(context: Text, warn: Optional[bool] = False) -> None:
		if warn:
			print(Fore.YELLOW + "Warning: " + context + Style.RESET_ALL)
		else:
			print(Fore.RED + "Error: " + context + Style.RESET_ALL)
			sys.exit(1)

class OpenGPT:
	@classmethod
	def __init__(self: type, provider: Text, type: Optional[Text] = "completion", options: Optional[Union[Dict[Text, Text], None]] = None) -> None:
		self.__LoadModels()
		self.__DIR: Text = os.getcwd()
		self.__TYPE: Text = type
		self.__OPTIONS: Union[Dict[Text, Text], None] = options
		self.__PROVIDER: Text = provider

		self.__Verifications()
		self.__MODULE: module = importlib.import_module(f"opengpt.models.{self.__TYPE}.{self.__PROVIDER}.model")
		self.__MODEL_CLASS: type = getattr(self.__MODULE, "Model")

		self.__model: type = None
		self.__InitializeModelClass()

	@classmethod
	def __InitializeModelClass(self: type) -> None:
		if not self.__OPTIONS is None:
			args: List[Text] = inspect.signature(self.__MODEL_CLASS.__init__)
			reqArgs: List[Text] = [
				param.name
				for param in args.parameters.values()
				if param.default == inspect.Parameter.empty
			]
			missingArgs: List[Text] = []

			for arg in reqArgs:
				if arg not in self.__OPTIONS:
					missingArgs.append(arg)

			if len(missingArgs) > 0:
				OpenGPTError.Print(f"Missing one of non-optional parameters: {', '.join(missingArgs)}")

			self.__model = self.__MODEL_CLASS(**self.__OPTIONS)
		else:
			self.__model = self.__MODEL_CLASS()

	@classmethod
	def __LoadModels(self: type) -> None:
		self.__DATA: Dict[Text, Text] = yaml.safe_load(open(self.__DIR + "/config.yml").read())

	@classmethod
	def __Verifications(self: type) -> None:
		exists: bool = False

		if self.__TYPE not in self.__DATA["models"]:
			OpenGPTError.Print(f"The type \"{self.__TYPE}\" not be founded. Try: {', '.join(self.__DATA['models'])}")

		if self.__PROVIDER not in self.__DATA["models"][self.__TYPE]:
			for type in self.__DATA["models"]:
				for model in self.__DATA["models"][type]:
					if model == self.__PROVIDER:
						rest: Text = f"Changing to \"{type}\"."
						OpenGPTError.Print(context=f"The provider \"{self.__PROVIDER}\" not is from type \"{self.__TYPE}\". " + rest, warn=True)
						self.__TYPE = type
						exists = True
		else:
			exists = True

		if not exists:
			OpenGPTError.Print(f"The provider \"{self.__PROVIDER}\" not exists.")

		incompletRest: Text = ''
		rest: Text = ''
		
		if self.__DATA["models"][self.__TYPE][self.__PROVIDER]["state"] == "incomplete":
			incompletRest = f"The provider \"{self.__PROVIDER}\" is incomplete. "
			if len(self.__DATA["models"][self.__TYPE][self.__PROVIDER]["bugs"]) > 0:
				rest = "In addition to having some bugs that have not yet been fixed."

		if len(rest) > 0 or len(incompletRest) > 0:
			OpenGPTError.Print(context=incompletRest + rest + " It is recommended not to use.", warn=True)

	@classmethod
	def __getattr__(self: type, name: Text) -> None:
		if hasattr(self.__model, name):
			attr: type = getattr(self.__model, name)
			if callable(attr):
				return lambda *args, **kwargs: attr(*args, **kwargs)
			return attr
		raise AttributeError(f"\"OpenGPT\" object has no attribute \"{name}\"")
